@@ -200,26 +200,26 @@ def endval_(*args):
 
 def pulse(*args):
     args = remove_nesting(args)
-    volume = parseExpression(args[0]) if type(parseExpression(args[0])) is str else parseExpression(args[0])[0]
+    volume = parseExpression(args[0])
 
-    first = None if len(args) == 1 else args[1][1]
+    first = None if len(args) == 1 else parseExpression(args[1])
 
-    interval = None if len(args) == 1 else args[2][1]
+    interval = None if len(args) <= 2 else parseExpression(args[2])
 
     if first == interval == None:
-        return '( ' + str(volume) + ' ) / self.dt'
+        return '(( ' + str(volume) + ' ) / self.dt)'
 
     if first == None:
         first = ' self.starttime '
 
     if interval == None:
-        return str(volume) + ' /self.dt if ' + str(first) + ' <= t else 0'
+        return '('+str(volume) + ' /self.dt if ' + str(first) + ' <= t else 0)'
 
     if int(interval) == 0:
-        return str(volume) + ' /self.dt if ' + str(first) + ' == t else 0'
+        return '('+ str(volume) + ' /self.dt if ' + str(first) + ' == t else 0)'
 
-    return str(volume) + '/ self.dt if ' + str(first) + ' <= t and ((t -' + str(first) + ') % ' + str(
-        interval) + ') == 0 else 0'
+    return '('+str(volume) + '/ self.dt if ' + str(first) + ' <= t and ((t -' + str(first) + ') % ' + str(
+        interval) + ') == 0 else 0)'
 
 
 def derivn_(*args):
@@ -244,7 +244,7 @@ def previous(*args):
 
     if initial:
 
-        return '(' + str(initial) + ') if t <= self.starttime else ' + '(' + str(body) + ')' if initial else str(body)
+        return '((' + str(initial) + ') if t <= self.starttime else ' + '(' + str(body) + '))' if initial else str(body)
     else:
 
         return str(body)
@@ -991,7 +991,7 @@ builtins = {
 
     # Test input builtins
     # http://www.iseesystems.com/Helpv10/Content/Reference/Builtins/Test_input_builtins.htm
-    'pulse': lambda *args: pulse(*args),
+    'pulse': lambda *args: pulse(args),
 
     'clocktime' : lambda *args : 'int(datetime.utcnow().timestamp())',
 
