@@ -333,10 +333,10 @@ grammar = Grammar(
 
     Keyword                 = if / then / else / and / or
     if                      =  ~"IF"i
-    then                    =  ~"THEN"i ' '
-    else                    =  ~"ELSE"i ' '
-    and                     =  ~"AND"i  ' '
-    or                      =  ~"OR"i  ' '
+    then                    =  ~"THEN"i
+    else                    =  ~"ELSE"i
+    and                     =  ~"AND"i
+    or                      =  ~"OR"i
 
 
     Name                    = ( ~"[\"]?" ~"[a-zA-Z0-9_%\$€£¥&§]"+ ~"[\"]"? )
